//by Marcel Hebert in Marcel's Workshop ©2021


#include <Nextion.h> // Nextion Library
#include <SoftwareSerial.h> // Library for Software Serial Port
#include <Keyboard.h> // Keyboard Emulation
#include <Adafruit_NeoPixel.h>
#define PIN            9
#define NUMPIXELS      4
Adafruit_NeoPixel pixels = Adafruit_NeoPixel(NUMPIXELS, PIN, NEO_GRB + NEO_KHZ800);
SoftwareSerial HMISerial(10, 11); // Nextion TX to pin 10 and RX to pin 11 of Arduino
const int mvm = 13; // Set pin for Mini Vibration Motor
const unsigned long eventInterval = 155;
unsigned long previousTime = 0;

// This part is used to declare all the objects we can receive from the Nextion Display (page, ID, Object name)
NexButton b0page1 = NexButton(1, 2, "b0"); // Ancestry
NexButton b1page1 = NexButton(1, 3, "b1"); // My Heritage
NexButton b2page1 = NexButton(1, 4, "b2"); // 23 & Me
NexButton b3page1 = NexButton(1, 5, "b3"); // BMS2000
NexButton b4page1 = NexButton(1, 6, "b4"); // Newpapers
NexButton b5page1 = NexButton(1, 7, "b5"); // Nos Origines
NexButton b6page1 = NexButton(1, 8, "b6"); // Genetic Affairs
NexButton b7page1 = NexButton(1, 9, "b7"); // Ged Match
NexButton b8page1 = NexButton(1, 10, "b8"); // DNA Painter
NexButton b9page1 = NexButton(1, 11, "b9"); // FTDNA
NexButton b10page1 = NexButton(1, 12, "b10"); // DNA Painter Chart
NexButton b11page1 = NexButton(1, 13, "b11"); // Facebook
NexButton b12page1 = NexButton(1, 14, "b12"); // Family Search
//NexButton b13page1 = NexButton(1, 15, "b13"); // Months - Page 2
//NexButton b14page1 = NexButton(1, 16, "b14"); // Char Map
//NexButton b15page1 = NexButton(1, 17, "b15"); // Larry Jones Ball Pattern System
//NexButton b16page1 = NexButton(1, 18, "b16"); // ALT+F4 (Danger)
NexButton b17page1 = NexButton(1, 19, "b17"); // Google
NexButton b18page1 = NexButton(1, 20, "b18"); // F5 aka Refresh
NexButton b19page1 = NexButton(1, 21, "b19"); // Cut
NexButton b20page1 = NexButton(1, 22, "b20"); // Copy
NexButton b21page1 = NexButton(1, 23, "b21"); // Paste
NexButton b22page1 = NexButton(1, 24, "b22"); // Undo
NexButton b23page1 = NexButton(1, 25, "b23"); // Redo
//NexButton b24page1 = NexButton(1, 26, "b24"); // Splash - Page 0 (Hidden in the botton left corner)

NexButton b0page2 = NexButton(2, 3, "b0"); // JAN
NexButton b1page2 = NexButton(2, 4, "b1"); // FEB
NexButton b2page2 = NexButton(2, 5, "b2"); // MAR
NexButton b3page2 = NexButton(2, 6, "b3"); // APR
NexButton b4page2 = NexButton(2, 7, "b4"); // MAY
NexButton b5page2 = NexButton(2, 8, "b5"); // JUN
NexButton b6page2 = NexButton(2, 9, "b6"); // JUL
NexButton b7page2 = NexButton(2, 10, "b7"); // AUG
NexButton b8page2 = NexButton(2, 11, "b8"); // SEP
NexButton b9page2 = NexButton(2, 12, "b9"); // OCT
NexButton b10page2 = NexButton(2, 13, "b10"); // NOV
NexButton b11page2 = NexButton(2, 14, "b11"); // DEC
//NexHotspot m0page2 = NexHotspot(2, 2, "m0"); // page1

//NexHotspot m0page3 = NexHotspot(3, 2, "m0"); // page1

NexButton b0page4 = NexButton(4, 3, "b0"); // ALT+F4
//NexHotspot m0page4 = NexHotspot(4, 2, "m0"); // page1

//NexButton b0page5 = NexButton(5, 29, "b0"); // page6

// This part is used to list the possible touchscreen events in an Array
NexTouch *nextion_touch_events[] =
{
  &b0page1, // Page 1 Hotspot
  &b1page1,
  &b2page1,
  &b3page1,
  &b4page1,
  &b5page1,
  &b6page1,
  &b7page1,
  &b8page1,
  &b9page1,
  &b10page1,
  &b11page1,
  &b12page1,
  //  &b13page1, Commmented out lines are for Nextion interal use only, no serial command declared.
  // &b14page1,
  //  &b15page1,
  &b17page1,
  &b18page1,
  &b19page1,
  &b20page1,
  &b21page1,
  &b22page1,
  &b23page1,
  // &b24page1,

  &b0page2, // Page 2 HotSpot
  &b1page2,
  &b2page2,
  &b3page2,
  &b4page2,
  &b5page2,
  &b6page2,
  &b7page2,
  &b8page2,
  &b9page2,
  &b10page2,
  &b11page2,
  // &m0page2,

  // &m0page3, //Page 3 HotSpot
  &b0page4, //Page 4 HotSpot
  //  &m0page4,
  // &b0page5, //Page 5 HotSpot
  //  &bt0page5,

  NULL // End string
};

// This part is for the different functions for Main Screen Touch events
void b0page1PushCallback(void *ptr)
{
  haptic(); //start Mini Vibration Motor and show lights in Green
  browser(); // Open the Prefered Browser, without this Windopws opens Edge (Yuck!)
  Keyboard.println ("ancestry.com/"); //URl requested
  holdUP(); //Give the computer time to open the web page, some pages are slower.
}

void b1page1PushCallback(void *ptr)
{
  haptic();
  browser();
  Keyboard.println ("myheritage.com");
  holdUP();
}

void b2page1PushCallback(void *ptr)
{
  haptic();
  browser();
  Keyboard.println ("https://www.23andme.com/");
  holdUP();
}

void b3page1PushCallback(void *ptr)
{
  haptic();
  browser();
  Keyboard.println ("http://www.bms2000.org/en");
  holdUP();
}

void b4page1PushCallback(void *ptr)
{
  haptic();
  browser();
  Keyboard.println ("https://www.newspapers.com/ ");
  holdUP();
}

void b5page1PushCallback(void *ptr)
{
  haptic();
  browser();
  Keyboard.println ("https://www.nosorigines.qc.ca/ ");
  holdUP();
}

void b6page1PushCallback(void *ptr)
{
  haptic();
  browser();
  Keyboard.println ("https://geneticaffairs.com/");
  holdUP();
}

void b7page1PushCallback(void *ptr)
{
  haptic();
  browser();
  Keyboard.println ("https://www.gedmatch.com/ ");
  holdUP();
}

void b8page1PushCallback(void *ptr)
{
  haptic();
  browser();
  Keyboard.println ("https://dnapainter.com/ ");
  holdUP();
}

void b9page1PushCallback(void *ptr)
{
  haptic();
  browser();
  Keyboard.println ("https://www.familytreedna.com/");
  holdUP();
}

void b10page1PushCallback(void *ptr)
{
  haptic();
  browser();
  Keyboard.println ("https://dnapainter.com/tools/sharedcmv4 ");
  holdUP();
}

void b11page1PushCallback(void *ptr)
{
  haptic();
  browser();
  Keyboard.println ("https://www.facebook.com/");
  holdUP();
}

void b12page1PushCallback(void *ptr)
{
  haptic();
  browser();
  Keyboard.println ("https://www.familysearch.org/en/");
  holdUP();
}

void b17page1PushCallback(void *ptr)
{
  haptic();
  browser();
  Keyboard.println ("https://www.google.com");
  holdUP();
}

void b18page1PushCallback(void *ptr)
{
  haptic();
  Keyboard.press (KEY_F5); //Refresh the current screen Not everythin supports Refresh, For example Notepad will insert the time and date stamp instead.
  Keyboard.releaseAll();
}

void b19page1PushCallback(void *ptr)
{
  haptic();
  Keyboard.press (KEY_LEFT_CTRL);
  Keyboard.press('x'); // Cut the Highlighted Text
  Keyboard.releaseAll();
}

void b20page1PushCallback(void *ptr)
{
  haptic();
  Keyboard.press (KEY_LEFT_CTRL);
  Keyboard.press('c');  // Copy the Highlighted Text
  Keyboard.releaseAll();
}

void b21page1PushCallback(void *ptr)
{
  haptic();
  Keyboard.press (KEY_LEFT_CTRL);
  Keyboard.press('v'); // Paste the Highlighted Text that was Cut or Copied
  Keyboard.releaseAll();
}

void b22page1PushCallback(void *ptr)
{
  haptic();
  Keyboard.press (KEY_LEFT_CTRL);
  Keyboard.press('z'); // UNDO, This only works if the program supports it
  Keyboard.releaseAll();
}

void b23page1PushCallback(void *ptr)
{
  haptic();
  Keyboard.press (KEY_LEFT_CTRL);
  Keyboard.press('y'); // REDO, This only works if the program supports it
  Keyboard.releaseAll();
}

void b0page2PushCallback(void *ptr)
{
  haptic(); //start Mini Vibration Motor and show lights in Green
  Keyboard.print ("Jan"); // Print Month in abbreviated from with NO line return
}
void b1page2PushCallback(void *ptr)
{
  haptic();
  Keyboard.print ("Feb");
}
void b2page2PushCallback(void *ptr)
{
  haptic();
  Keyboard.print ("Mar");
}
void b3page2PushCallback(void *ptr)
{
  haptic();
  Keyboard.print ("Apr");
}
void b4page2PushCallback(void *ptr)
{
  haptic();
  Keyboard.print ("May");
}
void b5page2PushCallback(void *ptr)
{
  haptic();
  Keyboard.print ("Jun");
}
void b6page2PushCallback(void *ptr)
{
  haptic();
  Keyboard.print ("Jul");
}
void b7page2PushCallback(void *ptr)
{
  haptic();
  Keyboard.print ("Aug");
}
void b8page2PushCallback(void *ptr)
{
  haptic();
  Keyboard.print ("Sep");
}
void b9page2PushCallback(void *ptr)
{
  haptic();
  Keyboard.print ("Oct");
}
void b10page2PushCallback(void *ptr)
{
  haptic();
  Keyboard.print ("Nov");
}
void b11page2PushCallback(void *ptr)
{
  haptic();
  Keyboard.print ("Dec");
}

//ALT F4 on Page 4
void b0page4PushCallback(void *ptr)
{
  haptic(); //start Mini Vibration Motor and show lights in Green
  Keyboard.press (KEY_LEFT_ALT);
  Keyboard.press (KEY_F4); //close program BE CAREFUL.
  Keyboard.releaseAll();
}

// Main Setup
void setup()
{
  pixels.begin(); // This initializes the NeoPixel library.

  HMISerial.begin(9600); // Start Software Serial at 9600bauds
  nexInit(); // Nextion Display initalize

  pinMode (mvm, OUTPUT); // Set MVM pin as output

  Keyboard.begin(); // Start Keyboard emulation

  // Link the touchscreen events to their relative functions in the code
  // attachPush for press events or attachPop for release events

  b0page1.attachPush(b0page1PushCallback); //Page 1 Hotspot---Ancestry
  b1page1.attachPush(b1page1PushCallback); //Page 1 Hotspot---My Heritage
  b2page1.attachPush(b2page1PushCallback); //Page 1 Hotspot---23 & Me
  b3page1.attachPush(b3page1PushCallback); //Page 1 Hotspot---BMS2000
  b4page1.attachPush(b4page1PushCallback); //Page 1 Hotspot---Newpapers
  b5page1.attachPush(b5page1PushCallback); //Page 1 Hotspot---Nos Origines
  b6page1.attachPush(b6page1PushCallback); //Page 1 Hotspot---Genetic Affairs
  b7page1.attachPush(b7page1PushCallback); //Page 1 Hotspot---Ged Match
  b8page1.attachPush(b8page1PushCallback); //Page 1 Hotspot---DNA Painter
  b9page1.attachPush(b9page1PushCallback); //Page 1 Hotspot---FTDNA
  b10page1.attachPush(b10page1PushCallback); //Page 1 Hotspot---DNA Painter Chart
  b11page1.attachPush(b11page1PushCallback); //Page 1 Hotspot---Facebook
  b12page1.attachPush(b12page1PushCallback); //Page 1 Hotspot---Family Search

  b0page4.attachPush(b0page4PushCallback); //Page 1 Hotspot---ALT-F4
  b17page1.attachPush(b17page1PushCallback); //Page 1 Hotspot---Google
  b18page1.attachPush(b18page1PushCallback); //Page 1 Hotspot--- F5
  b19page1.attachPush(b19page1PushCallback); //Page 1 Hotspot---Cut
  b20page1.attachPush(b20page1PushCallback); //Page 1 Hotspot---Copy
  b21page1.attachPush(b21page1PushCallback); //Page 1 Hotspot---Paste
  b22page1.attachPush(b22page1PushCallback); //Page 1 Hotspot---Undo
  b23page1.attachPush(b23page1PushCallback); //Page 1 Hotspot---Redo
  //  b24page1.attachPush(b24page1PushCallback); //Page 1 Hotspot---Splash - Page 0
  b0page2.attachPush(b0page2PushCallback); //Page 2 Hotspot---JAN
  b1page2.attachPush(b1page2PushCallback); //Page 2 Hotspot---FEB
  b2page2.attachPush(b2page2PushCallback); //Page 2 Hotspot---MAR
  b3page2.attachPush(b3page2PushCallback); //Page 2 Hotspot---APR
  b4page2.attachPush(b4page2PushCallback); //Page 2 Hotspot---MAY
  b5page2.attachPush(b5page2PushCallback); //Page 2 Hotspot---JUN
  b6page2.attachPush(b6page2PushCallback); //Page 2 Hotspot---JUL
  b7page2.attachPush(b7page2PushCallback); //Page 2 Hotspot---AUG
  b8page2.attachPush(b8page2PushCallback); //Page 2 Hotspot---SEP
  b9page2.attachPush(b9page2PushCallback); //Page 2 Hotspot---OCT
  b10page2.attachPush(b10page2PushCallback); //Page 2 Hotspot---NOV
  b11page2.attachPush(b11page2PushCallback); //Page 2 Hotspot---DEC
}
// Main Loop
void loop() {

  delay(100); // small delay before checking again for touch events

  nexLoop(nextion_touch_events); // Check for any touch event and run the associated function

  pixels.setPixelColor(0, pixels.Color(4, 4, 4)); // dim blue // Set all four lights to blue
  pixels.setPixelColor(1, pixels.Color(4, 4, 4)); // dim blue
  pixels.setPixelColor(2, pixels.Color(4, 4, 4)); // dim blue
  pixels.setPixelColor(3, pixels.Color(4, 4, 4)); // dim blue
  pixels.show();                                              // Show the Blue lights
  pixels.setPixelColor(0, pixels.Color(0, 10, 0)); //  Green  //Set all four lights to Green but don't show yet
  pixels.setPixelColor(1, pixels.Color(0, 10, 0)); //  Green
  pixels.setPixelColor(2, pixels.Color(0, 10, 0)); //  Green
  pixels.setPixelColor(3, pixels.Color(0, 10, 0)); //  Green
}

void haptic() {             //Declare the haptic subroutine
  pixels.show();  // Show Green lights that were set in Main Loop
  digitalWrite(mvm, HIGH);  // Vibrate motor
  delay (200);
  digitalWrite(mvm, LOW); // Stop vibration
  return;
}

void holdUP() {   // Wait for browser and URL to respond. Some pages load slower
  unsigned long currentTime = millis();
  if ( currentTime - previousTime >= eventInterval);
  previousTime = currentTime;
  return;
}

void browser() {             //Declare the Browser used, Windows will default to Edge (Yuck!)
  Keyboard.write (KEY_LEFT_GUI);
  delay(101);
  Keyboard.print ("chrome.exe");
  delay(301);
  Keyboard.write (KEY_RETURN);
  delay(451);
  return;
}
